@@ -1,38 +1,41 @@
+import logging
+import os
+from typing import List, Optional
+
+from qtpy import QtWidgets, QtCore
+
 from arkitekt_next.app.fakts import (
     build_device_code_fakts,
     build_redeem_fakts,
     build_token_fakts,
 )
-from arkitekt_next.utils import create_arkitekt_next_folder
-from fakts_next.models import Manifest
+from arkitekt_next.constants import DEFAULT_ARKITEKT_URL
+from arkitekt_next.init_registry import InitHookRegistry, get_default_init_hook_registry
+from arkitekt_next.qt.types import QtApp
 from arkitekt_next.service_registry import (
     ServiceBuilderRegistry,
     get_default_service_registry,
 )
-from arkitekt_next.constants import DEFAULT_ARKITEKT_URL
-from qtpy import QtWidgets, QtCore
-from typing import List, Optional
-import os
-import logging
-from arkitekt_next.qt.types import QtApp
+from arkitekt_next.utils import create_arkitekt_next_folder
+from fakts_next.models import Manifest
 
 
 def qt(
-    identifier: str,
+    identifier: str | None = None,
     version: str = "0.0.1",
     logo: Optional[str] = None,
     scopes: Optional[List[str]] = None,
     url: str = DEFAULT_ARKITEKT_URL,
-    headless: bool = False,
     log_level: str = "ERROR",
-    parent: Optional[QtWidgets.QWidget] = None,
+    parent: Optional[QtCore.QObject] = None,
     token: Optional[str] = None,
     no_cache: bool = False,
     redeem_token: Optional[str] = None,
     app_kind: str = "development",
-    registry: Optional[ServiceBuilderRegistry] = None,
+    service_registry: Optional[ServiceBuilderRegistry] = None,
     description: Optional[str] = None,
-    **kwargs,
+    instance_id: str = "main",
+    init_hook_registry: Optional[InitHookRegistry] = None,
 ) -> QtApp:
     """Creates a next app
 
@@ -100,7 +103,11 @@ def qt(
     NextApp
         A built app, that can be used to interact with the ArkitektNext server
     """
-    registry = registry or get_default_service_registry()
+    service_registry = service_registry or get_default_service_registry()
+    init_hook_registry = init_hook_registry or get_default_init_hook_registry()
+
+    if identifier is None:
+        identifier = __file__.split("/")[-1].replace(".py", "")
 
     url = os.getenv("FAKTS_URL", url)
     token = os.getenv("FAKTS_TOKEN", token)
@@ -110,9 +117,9 @@ def qt(
         identifier=identifier,
         scopes=scopes if scopes else ["openid"],
         logo=logo,
-        requirements=registry.get_requirements(),
-        description=description,
+        requirements=service_registry.get_requirements(),
     )
+
     if token:
         fakts_next = build_token_fakts(
             manifest=manifest,
@@ -122,16 +129,21 @@ def qt(
 
     elif redeem_token:
         fakts_next = build_redeem_fakts(
-            manifest=manifest, redeem_token=redeem_token, url=url
+            manifest=manifest,
+            redeem_token=redeem_token,
+            url=url,
         )
     else:
         fakts_next = build_device_code_fakts(
             manifest=manifest,
             url=url,
             no_cache=no_cache,
+            headless=False,
         )
 
-    params = kwargs
+    params = {
+        "instance_id": instance_id,
+    }
 
     create_arkitekt_next_folder(with_cache=True)
 
@@ -143,10 +155,13 @@ def qt(
         logging.basicConfig(level=log_level)
 
     app = QtApp(
+        parent=parent,
         fakts=fakts_next,
-        services=registry.build_service_map(fakts=fakts_next, params=params),
+        services=service_registry.build_service_map(fakts=fakts_next, params=params),
     )
 
+    init_hook_registry.run_all(app)
+
     app.enter()
 
     return app
